@@ -20,17 +20,17 @@ import argparse
 import collections
 import copy
 import functools
-import uuid
-
 
 import six
 from six.moves import range
 
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 
+from congress import data_types
 from congress.datalog import analysis
 from congress.datalog import base
-from congress.datalog.builtin import congressbuiltin
+from congress.datalog import builtin
 
 # set up appropriate antlr paths per python version and import runtime
 # import appropriate Lexer & Parser per python version
@@ -83,8 +83,8 @@ class Schema(object):
     def __contains__(self, tablename):
         return tablename in self.map
 
-    @classmethod
-    def col(self, cols):
+    @staticmethod
+    def _col(cols):
         # For Datasource tables, columns would be in the format -
         # {'name': 'colname', 'desc': 'description'}
         if len(cols) and isinstance(cols[0], dict):
@@ -92,6 +92,20 @@ class Schema(object):
         else:
             return [x for x in cols]
 
+    @staticmethod
+    def _type(cols):
+        # For Datasource tables, columns would be in the format -
+        # {'name': 'colname', 'desc': 'description',
+        #  'type': 'typename', 'nullable': True/False}
+        if len(cols) and isinstance(cols[0], dict):
+            return [data_types.TypeNullabilityTuple(
+                data_types.TypesRegistry.type_class(
+                    x.get('type', 'Scalar')),
+                x.get('nullable', True)) for x in cols]
+        else:
+            return [data_types.TypeNullabilityTuple(data_types.Scalar, True)
+                    for x in cols]
+
     def columns(self, tablename):
         """Returns the list of column names for the given TABLENAME.
 
@@ -100,7 +114,17 @@ class Schema(object):
         if tablename not in self.map.keys():
             return
         cols = self.map[tablename]
-        return Schema.col(cols)
+        return Schema._col(cols)
+
+    def types(self, tablename):
+        """Returns the list of column names for the given TABLENAME.
+
+        Return None if the tablename's columns are unknown.
+        """
+        if tablename not in self.map.keys():
+            return
+        cols = self.map[tablename]
+        return Schema._type(cols)
 
     def arity(self, tablename):
         """Returns the number of columns for the given TABLENAME.
@@ -139,7 +163,7 @@ class Schema(object):
                 schema_change = (tablename, val, True, th)
         else:
             if tablename not in self:
-                LOG.warning("Attempt to delete a non-existant rule: %s", item)
+                LOG.warning("Attempt to delete a nonexistent rule: %s", item)
             elif self.count[tablename] > 1:
                 self.count[tablename] -= 1
                 schema_change = (tablename, None, False, th)
@@ -783,10 +807,10 @@ class Literal (object):
 
     def is_builtin(self, check_arguments=True):
         if check_arguments:
-            return congressbuiltin.builtin_registry.is_builtin(
+            return builtin.builtin_registry.is_builtin(
                 self.table, len(self.arguments))
         else:
-            return congressbuiltin.builtin_registry.is_builtin(
+            return builtin.builtin_registry.is_builtin(
                 self.table)
 
     def tablename(self, default_service=None):
@@ -801,84 +825,96 @@ class Literal (object):
         self.table.drop_service()
         return self
 
-    def eliminate_column_references(self, theories, default_theory=None,
-                                    index=0, prefix=''):
-        """Expand column references to traditional datalog positional args.
+    def eliminate_column_references_and_pad_positional(
+            self, theories, default_theory=None, index=0, prefix=''):
+        """Expand column references to positional args and pad positional args.
 
-        Returns a new literal, unless no column references.
+        Expand column references to traditional datalog positional args.
+        Also pad positional args if too few are provided.
+        Returns a new literal. If no column reference, unless no schema found
+        for the table.
         """
         # TODO(ekcs): remove unused parameter: index
         # corner cases
-        if len(self.named_arguments) == 0:
-            return self
-        theory = literal_theory(self, theories, default_theory)
-        if theory is None or theory.schema is None:
-            raise exception.IncompleteSchemaException(
-                "Literal %s uses named arguments, but the "
-                "schema is unknown." % self)
-        if theory.kind != base.DATASOURCE_POLICY_TYPE:  # eventually remove
-            raise exception.PolicyException(
-                "Literal {} uses column references, but '{}' does not "
-                "reference a datasource policy.".format(self, theory.name))
-        schema = theory.schema
-        if self.table.table not in schema:
-            raise exception.IncompleteSchemaException(
-                "Literal {} uses unknown table {}.".format(
-                    str(self), str(self.table.table)))
+        if len(self.named_arguments) > 0:
+            theory = literal_theory(self, theories, default_theory)
+            if theory is None or theory.schema is None:
+                raise exception.IncompleteSchemaException(
+                    "Literal %s uses named arguments, but the "
+                    "schema is unknown." % self)
+            if theory.kind != base.DATASOURCE_POLICY_TYPE:  # eventually remove
+                raise exception.PolicyException(
+                    "Literal {} uses column references, but '{}' does not "
+                    "reference a datasource policy.".format(self, theory.name))
+            schema = theory.schema
+            if self.table.table not in schema:
+                raise exception.IncompleteSchemaException(
+                    "Literal {} uses unknown table {}.".format(
+                        str(self), str(self.table.table)))
 
-        # check if named arguments conflict with positional or named arguments
-        errors = []
-        term_index = {}
-        for col, arg in six.iteritems(self.named_arguments):
-            if isinstance(col, six.string_types):  # column name
-                index = schema.column_number(self.table.table, col)
-                if index is None:
-                    errors.append(exception.PolicyException(
-                        "In literal {} column name {} does not exist".format(
-                            str(self), col)))
-                    continue
-                if index < len(self.arguments):
-                    errors.append(exception.PolicyException(
-                        "In literal {} column name {} references position {},"
-                        " which is already provided by position.".format(
-                            str(self), col, index)))
-                if index in self.named_arguments:
-                    errors.append(exception.PolicyException(
-                        "In literal {} column name {} references position {}, "
-                        "which is also referenced by number.))".format(
-                            str(self), col, index)))
-                if index in term_index:
-                    # should have already caught this case above
-                    errors.append(exception.PolicyException(
-                        "In literal {}, column name {} references position {},"
-                        " which already has reference {}".format(
-                            str(self), col, index, str(term_index[index]))))
-                term_index[index] = arg
-            else:  # column number
-                if col >= schema.arity(self.table.table):
-                    errors.append(exception.PolicyException(
-                        "In literal {} column index {} is too large".format(
-                            str(self), col)))
-                if col < len(self.arguments):
-                    errors.append(exception.PolicyException(
-                        "In literal {} column index {} "
-                        " is already provided by position.".format(
-                            str(self), col)))
-                name = schema.column_name(self.table.table, col)
-                if name in self.named_arguments:
-                    errors.append(exception.PolicyException(
-                        "In literal {} column index {} references column {}, "
-                        "which is also referenced by name.))".format(
-                            str(self), col, name)))
-                if col in term_index:
-                    # should have already caught this case above
-                    errors.append(exception.PolicyException(
-                        "In literal {} column index {} already has a reference"
-                        " {}".format(str(self), col, str(term_index[col]))))
-                term_index[col] = arg
-        if errors:
-            raise exception.PolicyException(
-                " ".join(str(err) for err in errors))
+            # check if named arguments conflict with positional or named args
+            errors = []
+            term_index = {}
+            for col, arg in self.named_arguments.items():
+                if isinstance(col, six.string_types):  # column name
+                    index = schema.column_number(self.table.table, col)
+                    if index is None:
+                        errors.append(exception.PolicyException(
+                            "In literal {} column name {} does not "
+                            "exist".format(str(self), col)))
+                        continue
+                    if index < len(self.arguments):
+                        errors.append(exception.PolicyException(
+                            "In literal {} column name {} references position "
+                            "{}, which is already provided by "
+                            "position.".format(str(self), col, index)))
+                    if index in self.named_arguments:
+                        errors.append(exception.PolicyException(
+                            "In literal {} column name {} references position "
+                            "{}, which is also referenced by number.))".format(
+                                str(self), col, index)))
+                    if index in term_index:
+                        # should have already caught this case above
+                        errors.append(exception.PolicyException(
+                            "In literal {}, column name {} references "
+                            "position {}, which already has reference "
+                            "{}".format(str(self), col, index,
+                                        str(term_index[index]))))
+                    term_index[index] = arg
+                else:  # column number
+                    if col >= schema.arity(self.table.table):
+                        errors.append(exception.PolicyException(
+                            "In literal {} column index {} is too "
+                            "large".format(str(self), col)))
+                    if col < len(self.arguments):
+                        errors.append(exception.PolicyException(
+                            "In literal {} column index {} "
+                            " is already provided by position.".format(
+                                str(self), col)))
+                    name = schema.column_name(self.table.table, col)
+                    if name in self.named_arguments:
+                        errors.append(exception.PolicyException(
+                            "In literal {} column index {} references column "
+                            "{}, which is also referenced by name.))".format(
+                                str(self), col, name)))
+                    if col in term_index:
+                        # should have already caught this case above
+                        errors.append(exception.PolicyException(
+                            "In literal {} column index {} already has a "
+                            "reference {}".format(
+                                str(self), col, str(term_index[col]))))
+                    term_index[col] = arg
+            if errors:
+                raise exception.PolicyException(
+                    " ".join(str(err) for err in errors))
+        else:
+            theory = literal_theory(self, theories, default_theory)
+            if theory is None or theory.schema is None:
+                return self
+            schema = theory.schema
+            if self.table.table not in schema:
+                return self
+            term_index = {}
 
         # turn reference args into position args
         position_args = list(self.arguments)  # copy the original list
@@ -917,7 +953,7 @@ class Rule(object):
         self.body = body
         self.location = location
         self._hash = None
-        self.id = id or uuid.uuid4()
+        self.id = id or uuidutils.generate_uuid()
         self.name = name
         self.comment = comment
         self.original_str = original_str
@@ -1073,17 +1109,21 @@ class Rule(object):
     def is_update(self):
         return self.head.is_update()
 
-    def eliminate_column_references(self, theories, default_theory=None):
-        """Return version of SELF where all column references have been removed.
+    def eliminate_column_references_and_pad_positional(
+            self, theories, default_theory=None):
+        """Return version of SELF /w col refs removed and pos args padded.
 
+        All column references removed. Positional args padded up to required
+        length.
         Throws exception if RULE is inconsistent with schemas.
         """
         pre = self._unused_variable_prefix()
         heads = []
         for i in range(0, len(self.heads)):
-            heads.append(self.heads[i].eliminate_column_references(
-                theories, default_theory=default_theory,
-                index=i, prefix='%s%s' % (pre, i)))
+            heads.append(
+                self.heads[i].eliminate_column_references_and_pad_positional(
+                    theories, default_theory=default_theory,
+                    index=i, prefix='%s%s' % (pre, i)))
 
         body = []
         sorted_lits = sorted(self.body)
@@ -1092,9 +1132,10 @@ class Rule(object):
             lit_rank[sorted_lits[i]] = i
 
         for i in range(0, len(self.body)):
-            body.append(self.body[i].eliminate_column_references(
-                theories, default_theory=default_theory,
-                index=i, prefix='%s%s' % (pre, lit_rank[self.body[i]])))
+            body.append(
+                self.body[i].eliminate_column_references_and_pad_positional(
+                    theories, default_theory=default_theory,
+                    index=i, prefix='%s%s' % (pre, lit_rank[self.body[i]])))
 
         return Rule(heads, body, self.location, name=self.name,
                     comment=self.comment, original_str=self.original_str)
@@ -1365,7 +1406,7 @@ class RuleDependencyGraph(utility.BagGraph):
         # TODO(thinrichs): should be able to have global_tablename
         #   return a Tablename object and therefore build a graph
         #   of Tablename objects instead of strings.
-        if is_atom(formula):
+        if is_atom_like(formula):
             if include_atoms:
                 table = formula.table.global_tablename(theory)
                 nodes.add(table)
@@ -1410,22 +1451,23 @@ def find_subpolicy(rules, required_tables, prohibited_tables,
                    output_tables):
     """Return a subset of rules pertinent to the parameters.
 
-    :param rules is the collection of Datalog rules to analyze
-    :param required_tables is the set of tablenames that a rule must depend on.
-    :param prohibited_tables is the set of tablenames that a rule must
+    :param: rules is the collection of Datalog rules to analyze
+    :param: required_tables is the set of tablenames that a rule must depend on
+    :param: prohibited_tables is the set of tablenames that a rule must
            NOT depend on.
-    :param output_tables is the set of tablenames that all rules must support.
+    :param: output_tables is the set of tablenames that all rules must support.
 
     Table R depends on table T if T occurs in the
     body of a rule with R in the head, or T occurs in the body of a rule
     where R depends on the table in the head of that rule.
 
     The subset of RULES chosen has several properties:
-    i) if a chosen rule has table R in the head, then one of @output_tables
+
+    1. if a chosen rule has table R in the head, then one of @output_tables
        depends on R
-    ii) if a chosen rule has R in the head, then R does not depend on
-        any of @prohibited_tables
-    iii) if a chosen rule has R in the head, then R depends on at least
+    2. if a chosen rule has R in the head, then R does not depend on
+       any of @prohibited_tables
+    3. if a chosen rule has R in the head, then R depends on at least
          one of @required_tables.
     """
     def filter_output_definitions(rule_permitted):
@@ -1509,8 +1551,8 @@ def reorder_for_safety(rule):
         if lit.is_negated():
             target_vars = lit.variable_names()
         elif lit.is_builtin():
-            builtin = congressbuiltin.builtin_registry.builtin(lit.table)
-            target_vars = lit.arguments[0:builtin.num_inputs]
+            built = builtin.builtin_registry.builtin(lit.table)
+            target_vars = lit.arguments[0:built.num_inputs]
             target_vars = set([x.name for x in target_vars if x.is_variable()])
         else:
             # neither a builtin nor negated
@@ -1652,10 +1694,10 @@ def literal_schema(literal, theories, default_theory=None,
                    theory_assertion=None):
     """Return the schema that applies to LITERAL or None.
 
-    :param LITERAL is a Literal for which we want the schema
-    :param THEORIES is a dictionary mapping the name of the theory
+    :param: LITERAL is a Literal for which we want the schema
+    :param: THEORIES is a dictionary mapping the name of the theory
            to the theory object
-    :param DEFAULT_THEORY is the theory to use if no theory is
+    :param: DEFAULT_THEORY is the theory to use if no theory is
            recorded as part of LITERAL
     :returns: the schema that applies to LITERAL or None
     """
@@ -1668,10 +1710,10 @@ def literal_schema(literal, theories, default_theory=None,
 def literal_theory(literal, theories, default_theory=None):
     """Return the theory that applies to LITERAL or None.
 
-    :param LITERAL is a Literal for which we want the schema
-    :param THEORIES is a dictionary mapping the name of the theory
+    :param: LITERAL is a Literal for which we want the schema
+    :param: THEORIES is a dictionary mapping the name of the theory
            to the theory object
-    :param DEFAULT_THEORY is the theory to use if no theory is
+    :param: DEFAULT_THEORY is the theory to use if no theory is
            recorded as part of LITERAL
     :returns: the theory that applies to LITERAL or None
     """
@@ -1744,7 +1786,7 @@ def literal_schema_consistency(literal, theories, theory=None):
     arity = schema.arity(literal.table.table)
     if arity is not None and len(literal.arguments) != arity:
         return [exception.PolicyException(
-            "Literal {} contained {} arguments but only "
+            "Literal {} contained {} arguments but exactly "
             "{} arguments are permitted".format(
                 str(literal), len(literal.arguments), arity))]
 
@@ -1805,6 +1847,22 @@ def is_regular_rule(x):
     return (is_rule(x) and len(x.heads) == 1)
 
 
+def is_atom_rule(x):
+    return is_regular_rule(x) and len(x.body) == 0 and is_literal(x.heads[0])
+
+
+def is_literal_rule(x):
+    return is_regular_rule(x) and len(x.body) == 0 and is_literal(x.heads[0])
+
+
+def is_atom_like(x):
+    return is_atom(x) or is_atom_rule(x)
+
+
+def is_literal_like(x):
+    return is_literal(x) or is_literal_rule(x)
+
+
 def is_multi_rule(x):
     """Returns True if X is a rule with multiple heads."""
     return (is_rule(x) and len(x.heads) != 1)
@@ -2082,7 +2140,7 @@ class DatalogSyntax(object):
                 name = param.children[0].children[0].getText()
                 if name in reference_args:
                     errors.append(exception.PolicyException(
-                        "In atom {} two values for column name {} "
+                        "In atom {} multiple values for column name {} "
                         "were provided".format(atomstr, name)))
                 reference_args[name] = self.create_term(param.children[1])
             else:
@@ -2091,7 +2149,7 @@ class DatalogSyntax(object):
                 number = int(param.children[0].children[0].getText())
                 if number in reference_args:
                     errors.append(exception.PolicyException(
-                        "In atom {} two values for column number {} "
+                        "In atom {} multiple values for column number {} "
                         "were provided.".format(atomstr, str(number))))
                 reference_args[number] = self.create_term(param.children[1])
                 if number < len(position_args):
@@ -2237,6 +2295,17 @@ def print_tree(tree, text, kids, ind=0):
             print_tree(child, text, kids, ind + 1)
 
 
+def string_is_servicename(name):
+    """Returns True if @name can be a servicename in the policy language."""
+    if name == "builtin":   # 'builtin' is reserved for builtins
+        return False
+    try:
+        rules = parse("p :- %s:q()" % name)
+        return len(rules) == 1
+    except exception.PolicyException:
+        return False
+
+
 ##############################################################################
 # Mains
 ##############################################################################
