@@ -47,6 +47,9 @@ class Graph(object):
         def __eq__(self, other):
             return self.node == other.node and self.label == other.label
 
+        def __ne__(self, other):
+            return not self.__eq__(other)
+
         def __hash__(self):
             return hash(str(self))
 
@@ -300,7 +303,7 @@ class Graph(object):
 
         Node T is dependent on node T.
         Node T is dependent on node R if there is an edge from node S to T,
-            and S is dependent on R.
+        and S is dependent on R.
 
         Note that node T is dependent on node T even if T is not in the graph
         """
@@ -505,6 +508,9 @@ class OrderedSet(collections.MutableSet):
         else:
             return False
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
 
 class iterstr(object):
     """Lazily provides informal string representation of iterables.
