@@ -39,21 +39,21 @@ class DoctorDriver(datasource_driver.PushedDataSourceDriver):
     which means if you update the table with [] it will clear the table.
     One {} object in the list represents one row of the table.
 
-    request body:
-    [
-      {
-        "id": "0123-4567-89ab",
-        "time": "2016-02-22T11:48:55Z",
-        "type": "compute.host.down",
-        "details": {
-            "hostname": "compute1",
-            "status": "down",
-            "monitor": "zabbix1",
-            "monitor_event_id": "111"
-        }
-      },
-      .....
-    ]
+    request body::
+
+        [
+          {
+            "time": "2016-02-22T11:48:55Z",
+            "type": "compute.host.down",
+            "details": {
+                "hostname": "compute1",
+                "status": "down",
+                "monitor": "zabbix1",
+                "monitor_event_id": "111"
+            }
+          },
+          .....
+        ]
     """
 
     value_trans = {'translation-type': 'VALUE'}
@@ -81,8 +81,7 @@ class DoctorDriver(datasource_driver.PushedDataSourceDriver):
         'selector-type': 'DICT_SELECTOR',
         'objects-extract-fn': flatten_events,
         'field-translators':
-            ({'fieldname': 'id', 'translator': value_trans},
-             {'fieldname': 'time', 'translator': value_trans},
+            ({'fieldname': 'time', 'translator': value_trans},
              {'fieldname': 'type', 'translator': value_trans},
              {'fieldname': 'hostname', 'translator': value_trans},
              {'fieldname': 'status', 'translator': value_trans},
@@ -99,7 +98,9 @@ class DoctorDriver(datasource_driver.PushedDataSourceDriver):
     def get_datasource_info():
         result = {}
         result['id'] = 'doctor'
-        result['description'] = ('Datasource driver that interfaces with '
-                                 'OpenStack Compute aka nova.')
+        result['description'] = ('Datasource driver that allows external '
+                                 'systems to push data in accordance with '
+                                 'OPNFV Doctor Inspector southbound interface '
+                                 'specification.')
         result['config'] = {'persist_data': constants.OPTIONAL}
         return result
