@@ -15,22 +15,15 @@ from __future__ import division
 from __future__ import absolute_import
 
 import heatclient.v1.client as heatclient
-from keystoneauth1.identity import v2
-from keystoneauth1 import session
-import keystoneclient.v2_0.client as ksclient
 from oslo_log import log as logging
 
+from congress.datasources import constants
 from congress.datasources import datasource_driver
 from congress.datasources import datasource_utils as ds_utils
 
 LOG = logging.getLogger(__name__)
 
 
-def d6service(name, keys, inbox, datapath, args):
-    """This method is called by d6cage to create a dataservice instance."""
-    return HeatV1Driver(name, keys, inbox, datapath, args)
-
-
 class HeatV1Driver(datasource_driver.PollingDataSourceDriver,
                    datasource_driver.ExecutionDriver):
 
@@ -150,19 +143,15 @@ class HeatV1Driver(datasource_driver.PollingDataSourceDriver,
     TRANSLATORS = [stacks_translator, software_deployment_translator,
                    resources_translator, events_translator]
 
-    def __init__(self, name='', keys='', inbox=None, datapath=None, args=None):
-        super(HeatV1Driver, self).__init__(name, keys, inbox, datapath, args)
+    def __init__(self, name='', args=None):
+        super(HeatV1Driver, self).__init__(name, args=args)
         datasource_driver.ExecutionDriver.__init__(self)
         self.creds = args
-        auth = v2.Password(auth_url=self.creds['auth_url'],
-                           username=self.creds['username'],
-                           password=self.creds['password'],
-                           tenant_name=self.creds['tenant_name'])
-        sess = session.Session(auth=auth)
-        keystone = ksclient.Client(**self.creds)
-        endpoint = keystone.service_catalog.url_for(
-            service_type='orchestration', endpoint_type='publicURL')
-        self.heat = heatclient.Client(session=sess, endpoint=endpoint)
+        session = ds_utils.get_keystone_session(self.creds)
+        endpoint = session.get_endpoint(service_type='orchestration',
+                                        interface='publicURL')
+        self.heat = heatclient.Client(session=session, endpoint=endpoint)
+        self.initialize_update_methods()
         self._init_end_start_poll()
 
     @staticmethod
@@ -172,28 +161,27 @@ class HeatV1Driver(datasource_driver.PollingDataSourceDriver,
         result['description'] = ('Datasource driver that interfaces with'
                                  ' OpenStack orchestration aka heat.')
         result['config'] = ds_utils.get_openstack_required_config()
+        result['config']['lazy_tables'] = constants.OPTIONAL
         result['secret'] = ['password']
         return result
 
-    def update_from_datasource(self):
-        """Called when it is time to pull new data from this datasource."""
-        LOG.debug("Grabbing Heat Stacks")
-        stacks = self.heat.stacks.list()
-        self._translate_stacks({'stacks': stacks})
+    def initialize_update_methods(self):
+        stacks_method = lambda: self._translate_stacks(
+            {'stacks': self.heat.stacks.list()})
+        self.add_update_method(stacks_method, self.stacks_translator)
 
-        LOG.debug("Grabbing resources")
-        stacks = self.heat.stacks.list()
-        resources = self._get_resources(stacks)
-        self._translate_resources(resources)
+        resources_method = lambda: self._translate_resources(
+            self._get_resources(self.heat.stacks.list()))
+        self.add_update_method(resources_method, self.resources_translator)
 
-        LOG.debug("Grabbing events")
-        stacks = self.heat.stacks.list()
-        events = self._get_events(stacks)
-        self._translate_events(events)
+        events_method = lambda: self._translate_events(
+            self._get_events(self.heat.stacks.list()))
+        self.add_update_method(events_method, self.events_translator)
 
-        LOG.debug("Grabbing the software deployments")
-        deployments = {'deployments': self.heat.software_deployments.list()}
-        self._translate_software_deployment(deployments)
+        deployments_method = lambda: self._translate_software_deployment(
+            {'deployments': self.heat.software_deployments.list()})
+        self.add_update_method(deployments_method,
+                               self.software_deployment_translator)
 
     def _get_resources(self, stacks):
         rval = []
