@@ -11,32 +11,32 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-#
-
-from oslo_log import log as logging
-from oslo_serialization import jsonutils as json
-
-LOG = logging.getLogger(__name__)
-
 import copy
-from oslo_config import cfg
-import six
+import six  # thirdparty import first because needed in import of Queue/queue
 import time
 if six.PY2:
     import Queue as queue_package
 else:
     import queue as queue_package
 
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_serialization import jsonutils as json
+
+from congress import exception
+
+LOG = logging.getLogger(__name__)
+
 
 class DataServiceInfo(object):
     """Metadata for DataService on the DSE.
 
     Attributes:
-        service_id: The ID of the service.
-        node_id: The ID of the node the service is running on.
-        published_tables: List of table IDs published by this service.
-        subscribed_tables: List of table IDs this service subscribes to.
-        rpc_endpoints_info: List of RPC endpoints exposed by this service.
+    - service_id: The ID of the service.
+    - node_id: The ID of the node the service is running on.
+    - published_tables: List of table IDs published by this service.
+    - subscribed_tables: List of table IDs this service subscribes to.
+    - rpc_endpoints_info: List of RPC endpoints exposed by this service.
     """
     MARSHALL_ATTRS = set(['service_id', 'node_id', 'published_tables',
                           'subscribed_tables', 'rpc_endpoints_info'])
@@ -91,17 +91,16 @@ class DataService(object):
     used for all inter-service communication.
 
     Attributes:
-        service_id: A unique ID of the service.
-        _published_tables_with_subscriber: A set of tables published by self
-            that has subscribers
+
+    - service_id: A unique ID of the service.
+    - _published_tables_with_subscriber: A set of tables published by self
+      that has subscribers
     """
 
     # TODO(pballand): make default methods for pub/subscribed tables
     def __init__(self, service_id):
         # Note(ekcs): temporary setting to disable use of diffs and sequencing
         #   to avoid muddying the process of a first dse2 system test.
-        # TODO(ekcs,dse2): remove when differential update is standard
-        self.always_snapshot = True
         self.service_id = service_id
         self.node = None
         self._rpc_server = None
@@ -151,7 +150,7 @@ class DataService(object):
             service_id=self.service_id,
             node_id=self.node.node_id if self.node else None,
             published_tables=None,
-            subscribed_tables=None,
+            subscribed_tables=self._published_tables_with_subscriber,
             rpc_endpoints_info=None,
             )
 
@@ -205,16 +204,6 @@ class DataService(object):
     def get_datasource(self, datasource_id):
         return self.node.get_datasource(datasource_id)
 
-    # Will be removed once the reference of node exists in api
-    # Note(thread-safety): blocking function
-    def add_datasource(self, **kwargs):
-        return self.node.add_datasource(**kwargs)
-
-    # Will be removed once the reference of node exists in api
-    # Note(thread-safety): blocking function
-    def delete_datasource(self, datasource):
-        return self.node.delete_datasource(datasource)
-
     # Will be removed once the reference of node exists in api
     # Note(thread-safety): blocking function
     def get_drivers_info(self, *args):
@@ -236,22 +225,24 @@ class DataService(object):
         return self.node.make_datasource_dict(*args, **kwargs)
 
     # Note(thread-safety): blocking function
-    def publish(self, table, data, use_snapshot=True):
-        if self.always_snapshot:
-            self.node.publish_table(self.service_id, table, data)
-            return
+    def publish(self, table, data, use_snapshot=False):
+        LOG.debug('Publishing table %s', table)
+        LOG.trace('Parameters: table: %s, data: %s, use_snapshot: %s',
+                  table, data, use_snapshot)
+        LOG.trace('Last published data %s', self._last_published_data)
+        data = set(data)  # make a copy to avoid co-reference
 
         def get_differential_and_set_last_published_data():
             if table in self._last_published_data:
-                to_add = list(
-                    set(data) - set(self._last_published_data[table]))
-                to_del = list(
-                    set(self._last_published_data[table]) - set(data))
+                LOG.trace('Diff against last published data %s',
+                          self._last_published_data[table])
+                to_add = list(data - self._last_published_data[table])
+                to_del = list(self._last_published_data[table] - data)
                 self._last_published_data[table] = data
             else:
-                self._last_published_data[table] = data
-                to_add = data
+                to_add = list(data)
                 to_del = []
+                self._last_published_data[table] = data
             return [to_add, to_del]
 
         def increment_get_seqnum():
@@ -263,6 +254,7 @@ class DataService(object):
 
         if not use_snapshot:
             data = get_differential_and_set_last_published_data()
+            LOG.debug('Differential data to publish %s', data)
             if len(data[0]) == 0 and len(data[1]) == 0:
                 return
 
@@ -272,16 +264,15 @@ class DataService(object):
 
     # Note(thread-safety): blocking function
     def subscribe(self, service, table):
-        if self.always_snapshot:
+        try:
             # Note(thread-safety): blocking call
-            data = self.node.subscribe_table(self.service_id, service, table)
-            self.receive_data(service, table, data, is_snapshot=True)
-            return
-        # Note(thread-safety): blocking call
-        (seqnum, data) = self.node.subscribe_table(
-            self.service_id, service, table)
-        self.receive_data_sequenced(
-            service, table, data, seqnum, is_snapshot=True)
+            (seqnum, data) = self.node.subscribe_table(
+                self.service_id, service, table)
+            self.receive_data_sequenced(
+                service, table, data, seqnum, is_snapshot=True)
+        except exception.NotFound:
+            LOG.info("Service '%s' unresponsive. '%s:%s' subscribed but data "
+                     "not yet initialized.", service, service, table)
 
     def unsubscribe(self, service, table):
         # Note(thread-safety): it is important to make sure there are no
@@ -453,18 +444,18 @@ class DataService(object):
     def subscriber_list(self):
         """Method that returns subscribers list.
 
-        This feature is duplicated in the distributed architecture. So the
-        method is defined only for backward compatibility.
+        It returns list of services subscribed to this service data.
         """
-        LOG.info('subscriber_list is duplicated in the new architecture.')
-        return []
+        return self.node.get_subscribers(self.service_id)
 
     def get_last_published_data_with_seqnum(self, table):
         """Method that returns the current seqnum & data for given table."""
         if table not in self.sender_seqnums:
             self.sender_seqnums[table] = 0
-            self._last_published_data[table] = self.get_snapshot(table)
-        return (self.sender_seqnums[table], self._last_published_data[table])
+            self._last_published_data[table] = set(self.get_snapshot(table))
+            # make a copy to avoid co-reference
+        return (self.sender_seqnums[table],
+                list(self._last_published_data[table]))
 
     def get_snapshot(self, table):
         """Method that returns the current data for the given table.
