@@ -320,7 +320,7 @@ class DseNode(object):
             except (messaging_exceptions.MessagingTimeout,
                     messaging_exceptions.MessageDeliveryFailure):
                 msg = "service '%s' could not be found"
-                raise exception.NotFound(msg % service_id)
+                raise exception.RpcTargetNotFound(msg % service_id)
         if kwargs is None:
             kwargs = {}
         try:
@@ -356,7 +356,7 @@ class DseNode(object):
             kwargs = {}
         if not self.is_valid_service(service_id):
             msg = "service '%s' is not a registered service"
-            raise exception.NotFound(msg % service_id)
+            raise exception.RpcTargetNotFound(msg % service_id)
 
         target = self.service_rpc_target(service_id, fanout=True)
         LOG.trace("<%s> Casting RPC '%s' on %s", self.node_id, method, target)
@@ -526,7 +526,7 @@ class DseNode(object):
 
     @classmethod
     def load_custom_drivers(cls):
-        cdist = pkg_resources.get_distribution('congress')
+        cdist = pkg_resources.get_distribution('openstack-congress')
         ext_list = []
         for driver in cfg.CONF.custom_driver_endpoints:
             try:
