@@ -219,8 +219,8 @@ class TestNeutronV2Driver(base.TestCase):
                  {u'direction': u'egress',
                   u'ethertype': u'IPv4',
                   u'id': u'1d943e83-e4e6-472a-9655-f74eb22f3668',
-                  u'port_range_max': None,
-                  u'port_range_min': None,
+                  u'port_range_max': 22,
+                  u'port_range_min': 11,
                   u'protocol': None,
                   u'remote_group_id': None,
                   u'remote_ip_prefix': None,
@@ -316,16 +316,16 @@ class TestNeutronV2Driver(base.TestCase):
                 ('3c0eb3a3-4d16-4b1b-b327-44417182d0bb',
                  'feee0a965cc34274917fb753623dd57d', 'private-subnet',
                  '63ce8fbb-12e9-4ecd-9b56-1bbf8b51217d', 4, '10.0.0.0/24',
-                 '10.0.0.1', 'True', 'None', 'None'),
+                 '10.0.0.1', True, None, None),
                 ('aa9ad4f7-baf0-4a41-85c3-1cc8a3066db6', '',
                  'meta-f42dc4f1-f371-48cc-95be-cf1b97112ab8',
                  '6743ff85-2cfd-48a7-9d3f-472cd418783e', 4,
                  '169.254.169.252/30',
-                 '169.254.169.253', 'True', 'None', 'None'),
+                 '169.254.169.253', True, None, None),
                 ('10d20df9-e8ba-4756-ba30-d573ceb2e99a',
                  'feee0a965cc34274917fb753623dd57d', '',
                  'ecdea1af-7197-43c8-b3b0-34d90f72a2a8', 4, '1.1.1.0/24',
-                 '1.1.1.1', 'True', 'None', 'None')]),
+                 '1.1.1.1', True, None, None)]),
             'floating_ips': set([
                 ("2f245a7b-796b-4f26-9cf9-9e82d248fda7",
                  "d23abc8d-2991-4a55-ba98-2aaea84cc72f",
@@ -333,68 +333,68 @@ class TestNeutronV2Driver(base.TestCase):
                  "376da547-b977-4cfe-9cba-275c80debf57", "10.0.0.3",
                  "172.24.4.228", "ce705c24-c1ef-408a-bda3-7bbd946164ab",
                  "ACTIVE"),
-                ("61cea855-49cb-4846-997d-801b70c71bdd", 'None',
+                ("61cea855-49cb-4846-997d-801b70c71bdd", None,
                  "4969c491a3c74ee4af974e6d800c62de",
-                 "376da547-b977-4cfe-9cba-275c80debf57", 'None',
-                 "172.24.4.227", 'None', "DOWN")]),
+                 "376da547-b977-4cfe-9cba-275c80debf57", None,
+                 "172.24.4.227", None, "DOWN")]),
             'routers':
                 set([('f42dc4f1-f371-48cc-95be-cf1b97112ab8',
-                      'feee0a965cc34274917fb753623dd57d', 'DOWN', 'True',
-                      'myrouter', 'False')]),
+                      'feee0a965cc34274917fb753623dd57d', 'DOWN', True,
+                      'myrouter', False)]),
             'dns_nameservers':
                 set([('3c0eb3a3-4d16-4b1b-b327-44417182d0bb', '8.8.8.8')]),
             'security_group_rules':
                 set([('e0239062-4243-4798-865f-7055f03786d6',
                       'e70cf243-3389-4f80-82dc-92a3ec1f2d2a',
-                      'feee0a965cc34274917fb753623dd57d', 'None', 'egress',
-                      'IPv4', 'None', 'None', 'None', 'None'),
+                      'feee0a965cc34274917fb753623dd57d', None, 'egress',
+                      'IPv4', None, None, None, None),
                      ('a268fc32-1a59-4154-9a7c-f453ef92560c',
-                      'ed7fd9f6-e390-448a-9f5f-8dd4659282f7', '', 'None',
-                      'egress', 'IPv6', 'None', 'None', 'None', 'None'),
+                      'ed7fd9f6-e390-448a-9f5f-8dd4659282f7', '', None,
+                      'egress', 'IPv6', None, None, None, None),
                      ('a268fc32-1a59-4154-9a7c-f453ef92560c',
-                      '1d943e83-e4e6-472a-9655-f74eb22f3668', '', 'None',
-                      'egress', 'IPv4', 'None', 'None', 'None', 'None'),
+                      '1d943e83-e4e6-472a-9655-f74eb22f3668', '', None,
+                      'egress', 'IPv4', None, 11, 22, None),
                      ('a268fc32-1a59-4154-9a7c-f453ef92560c',
                       '30be5ee1-5b0a-4929-aca5-0c25f1c6b733', '',
                       'a268fc32-1a59-4154-9a7c-f453ef92560c', 'ingress',
-                      'IPv4', 'None', 'None', 'None', 'None'),
+                      'IPv4', None, None, None, None),
                      ('e0239062-4243-4798-865f-7055f03786d6',
                       '8a81fecc-ecc7-48ca-bccc-195799667e23',
                       'feee0a965cc34274917fb753623dd57d',
                       'e0239062-4243-4798-865f-7055f03786d6', 'ingress',
-                      'IPv6', 'None', 'None', 'None', 'None'),
+                      'IPv6', None, None, None, None),
                      ('a268fc32-1a59-4154-9a7c-f453ef92560c',
                       '639995b8-c3ac-44a3-a4f3-c74f9172ad54', '',
                       'a268fc32-1a59-4154-9a7c-f453ef92560c', 'ingress',
-                      'IPv6', 'None', 'None', 'None', 'None'),
+                      'IPv6', None, None, None, None),
                      ('e0239062-4243-4798-865f-7055f03786d6',
                       '8f4d9e99-1fe8-4816-9f07-c4ecddea9427',
                       'feee0a965cc34274917fb753623dd57d',
                       'e0239062-4243-4798-865f-7055f03786d6',
-                      'ingress', 'IPv4', 'None', 'None', 'None', 'None'),
+                      'ingress', 'IPv4', None, None, None, None),
                      ('e0239062-4243-4798-865f-7055f03786d6',
                       'eca1df0f-b222-4208-8f96-8a8024fd6834',
-                      'feee0a965cc34274917fb753623dd57d', 'None', 'egress',
-                      'IPv6', 'None', 'None', 'None', 'None')]),
+                      'feee0a965cc34274917fb753623dd57d', None, 'egress',
+                      'IPv6', None, None, None, None)]),
                 'ports':
                     set([('c58c3246-6c2e-490a-b4d9-3b8d5191b465',
                           'feee0a965cc34274917fb753623dd57d', '',
                           '63ce8fbb-12e9-4ecd-9b56-1bbf8b51217d',
-                          'fa:16:3e:08:31:6e', 'True', 'DOWN',
+                          'fa:16:3e:08:31:6e', True, 'DOWN',
                           'f42dc4f1-f371-48cc-95be-cf1b97112ab8',
                           'network:router_interface'),
                          ('87f8933a-9582-48d8-ad16-9abf6e545002', '', '',
                           '6743ff85-2cfd-48a7-9d3f-472cd418783e',
-                          'fa:16:3e:b7:78:e8', 'True', 'DOWN',
+                          'fa:16:3e:b7:78:e8', True, 'DOWN',
                           'f42dc4f1-f371-48cc-95be-cf1b97112ab8',
                           'network:router_interface'),
                          ('eb50003b-a081-4533-92aa-1cbd97f526a8',
                           'feee0a965cc34274917fb753623dd57d', '',
                           '63ce8fbb-12e9-4ecd-9b56-1bbf8b51217d',
-                          'fa:16:3e:af:56:fa', 'True', 'DOWN', '', ''),
+                          'fa:16:3e:af:56:fa', True, 'DOWN', '', ''),
                          ('04627c85-3553-436c-a7c5-0a64f5b87bb9', '', '',
                           'ecdea1af-7197-43c8-b3b0-34d90f72a2a8',
-                          'fa:16:3e:f3:19:e5', 'True', 'DOWN',
+                          'fa:16:3e:f3:19:e5', True, 'DOWN',
                           'f42dc4f1-f371-48cc-95be-cf1b97112ab8',
                           'network:router_gateway')]),
                 'allocation_pools':
@@ -412,7 +412,7 @@ class TestNeutronV2Driver(base.TestCase):
                           'e0239062-4243-4798-865f-7055f03786d6')]),
                 'external_gateway_infos':
                     set([('f42dc4f1-f371-48cc-95be-cf1b97112ab8',
-                          'ecdea1af-7197-43c8-b3b0-34d90f72a2a8', 'True')]),
+                          'ecdea1af-7197-43c8-b3b0-34d90f72a2a8', True)]),
                 'fixed_ips':
                     set([('c58c3246-6c2e-490a-b4d9-3b8d5191b465', '10.0.0.1',
                           '3c0eb3a3-4d16-4b1b-b327-44417182d0bb'),
@@ -426,10 +426,10 @@ class TestNeutronV2Driver(base.TestCase):
                 'networks':
                     set([('ecdea1af-7197-43c8-b3b0-34d90f72a2a8',
                           'feee0a965cc34274917fb753623dd57d', 'public',
-                          'ACTIVE', 'True', 'False'),
+                          'ACTIVE', True, False),
                          ('63ce8fbb-12e9-4ecd-9b56-1bbf8b51217d',
                           'feee0a965cc34274917fb753623dd57d', 'private',
-                          'ACTIVE', 'True', 'False')]),
+                          'ACTIVE', True, False)]),
                 'security_groups':
                     set([('e0239062-4243-4798-865f-7055f03786d6',
                           'feee0a965cc34274917fb753623dd57d', 'default',
@@ -483,3 +483,62 @@ class TestNeutronV2Driver(base.TestCase):
         self.driver.execute('connectNetwork', api_args)
 
         self.assertEqual(expected_ans, neutron_client.testkey)
+
+    def test_update_resource_attrs(self):
+        args = {'positional': ['port', '1', 'key1', 'val1']}
+        action_args = {'named': {'port': '1',
+                                 'body': {'port': {'key1': 'val1'}}}}
+        with mock.patch.object(self.driver, '_execute_api') as mock_ea:
+            self.driver.update_resource_attrs(args)
+            mock_ea.assert_called_with(self.driver.neutron, 'update_port',
+                                       action_args)
+
+    def test_attach_new_port_security_group(self):
+        port_info = {u'port': self.mock_ports['ports'][3]}
+        port_id = u'eb50003b-a081-4533-92aa-1cbd97f526a8'
+        existing_sgroup_id = u'e0239062-4243-4798-865f-7055f03786d6'
+        new_sgroup_id = u'a268fc32-1a59-4154-9a7c-f453ef92560c'
+        new_sgroup_args = {'positional': [port_id, new_sgroup_id]}
+        self.driver.neutron.show_port.return_value = port_info
+
+        self.driver.attach_port_security_group(new_sgroup_args)
+        self.driver.neutron.show_port.assert_called_once_with(port_id)
+        self.driver.neutron.update_port.assert_called_once_with(
+            port_id,
+            {'port': {'security_groups': [existing_sgroup_id, new_sgroup_id]}})
+
+    def test_attach_existing_port_security_group(self):
+        port_info = {u'port': self.mock_ports['ports'][3]}
+        port_id = u'eb50003b-a081-4533-92aa-1cbd97f526a8'
+        existing_sgroup_id = u'e0239062-4243-4798-865f-7055f03786d6'
+        existing_sgroup_args = {
+            'positional': [port_id, existing_sgroup_id]}
+        self.driver.neutron.show_port.return_value = port_info
+
+        self.driver.attach_port_security_group(existing_sgroup_args)
+        self.driver.neutron.show_port.assert_called_once_with(port_id)
+        self.driver.neutron.update_port.assert_not_called()
+
+    def test_detach_new_port_security_group(self):
+        port_info = {u'port': self.mock_ports['ports'][3]}
+        port_id = u'eb50003b-a081-4533-92aa-1cbd97f526a8'
+        new_sgroup_id = u'a268fc32-1a59-4154-9a7c-f453ef92560c'
+        new_sgroup_args = {'positional': [port_id, new_sgroup_id]}
+        self.driver.neutron.show_port.return_value = port_info
+
+        self.driver.detach_port_security_group(new_sgroup_args)
+        self.driver.neutron.show_port.assert_called_once_with(port_id)
+        self.driver.neutron.update_port.assert_not_called()
+
+    def test_detach_existing_port_security_group(self):
+        port_info = {u'port': self.mock_ports['ports'][3]}
+        port_id = u'eb50003b-a081-4533-92aa-1cbd97f526a8'
+        existing_sgroup_id = u'e0239062-4243-4798-865f-7055f03786d6'
+        existing_sgroup_args = {
+            'positional': [port_id, existing_sgroup_id]}
+        self.driver.neutron.show_port.return_value = port_info
+
+        self.driver.detach_port_security_group(existing_sgroup_args)
+        self.driver.neutron.show_port.assert_called_once_with(port_id)
+        self.driver.neutron.update_port.assert_called_once_with(
+            port_id, {'port': {'security_groups': []}})
