@@ -61,9 +61,10 @@ class TestNovaDriver(base.TestCase):
             flavor_id = t[7]
             zone = t[8]
             host_name = t[9]
-            self.assertIn(id, [1234, 5678, 9012])
+            created_at = t[10]
+            self.assertIn(id, ['1234', '5678', '9012'])
             # see congress.datasources.tests.unit.fakes for actual values
-            if id == 1234:
+            if id == '1234':
                 self.assertEqual("sample-server", name)
                 self.assertEqual("e4d909c290d0fb1ca068ffaddf22cbd0", host_id)
                 self.assertEqual("BUILD", status)
@@ -71,12 +72,13 @@ class TestNovaDriver(base.TestCase):
                                  user_id)
                 self.assertEqual("50e14867-7c64-4ec9-be8d-ed2470ca1d24",
                                  tenant_id)
-                self.assertEqual(2, image_id)
-                self.assertEqual(1, flavor_id)
+                self.assertEqual('2', image_id)
+                self.assertEqual('1', flavor_id)
+                self.assertEqual('2019-02-26T08:48:15Z', created_at)
                 self.assertEqual('default', zone)
                 self.assertEqual('host1', host_name)
 
-            elif id == 5678:
+            elif id == '5678':
                 self.assertEqual("sample-server2", name)
                 self.assertEqual("9e107d9d372bb6826bd81d3542a419d6", host_id)
                 self.assertEqual("ACTIVE", status)
@@ -84,12 +86,13 @@ class TestNovaDriver(base.TestCase):
                                  user_id)
                 self.assertEqual("50e14867-7c64-4ec9-be8d-ed2470ca1d24",
                                  tenant_id)
-                self.assertEqual(2, image_id)
-                self.assertEqual(1, flavor_id)
-                self.assertEqual('None', zone)
-                self.assertEqual('None', host_name)
+                self.assertEqual('2', image_id)
+                self.assertEqual('1', flavor_id)
+                self.assertEqual('2019-02-26T08:48:15Z', created_at)
+                self.assertEqual(None, zone)
+                self.assertEqual(None, host_name)
 
-            elif id == 9012:
+            elif id == '9012':
                 self.assertEqual("sample-server3", name)
                 self.assertEqual("9e107d9d372bb6826bd81d3542a419d6", host_id)
                 self.assertEqual("ACTIVE", status)
@@ -97,11 +100,29 @@ class TestNovaDriver(base.TestCase):
                                  user_id)
                 self.assertEqual("50e14867-7c64-4ec9-be8d-ed2470ca1d24",
                                  tenant_id)
-                self.assertEqual(2, image_id)
-                self.assertEqual(1, flavor_id)
+                self.assertEqual('2', image_id)
+                self.assertEqual('1', flavor_id)
+                self.assertEqual('2019-02-26T08:48:15Z', created_at)
                 self.assertEqual('foo', zone)
                 self.assertEqual('host2', host_name)
 
+        server_tag_tuples = self.driver.state[self.driver.TAGS]
+
+        self.assertEqual(len(server_tag_tuples), 5)
+
+        self.assertIn(('5678', 'tag1'), server_tag_tuples)
+        self.assertIn(('5678', 'tag2'), server_tag_tuples)
+
+        self.assertIn(('9012', 'tag1'), server_tag_tuples)
+        self.assertIn(('9012', 'tag2'), server_tag_tuples)
+        self.assertIn(('9012', 'tag3'), server_tag_tuples)
+
+        server_addresses_tuples = self.driver.state[self.driver.ADDRESSES]
+        self.assertEqual(len(server_addresses_tuples), 1)
+        self.assertIn(('1234', 'net_mgmt', '192.168.0.60', 4, '11:11:11:11',
+                       'fixed'),
+                      server_addresses_tuples)
+
     def test_flavors(self):
         flavor_raw = self.nova.flavors.list(detailed=True)
         self.driver._translate_flavors(flavor_raw)
@@ -120,13 +141,13 @@ class TestNovaDriver(base.TestCase):
             ephemeral = f[5]
             rxtx_factor = f[6]
 
-            self.assertIn(id, [1, 2, 3, 4])
+            self.assertIn(id, ['1', '2', '3', '4'])
 
             # {'id': 1, 'name': '256 MB Server', 'ram': 256, 'disk': 10,
             # 'vcpus' : 1, 'OS-FLV-EXT-DATA:ephemeral': 10,
             # 'os-flavor-access:is_public': True, 'rxtx_factor' : 1.0,
             # 'links': {}},
-            if id == 1:
+            if id == '1':
                 self.assertEqual('256 MB Server', name)
                 self.assertEqual(256, ram)
                 self.assertEqual(10, disk)
@@ -137,7 +158,7 @@ class TestNovaDriver(base.TestCase):
             #  'vcpus' :2, 'OS-FLV-EXT-DATA:ephemeral': 20,
             #  'os-flavor-access:is_public': False, 'rxtx_factor' : 1.0,
             #  'links': {}},
-            elif id == 2:
+            elif id == '2':
                 self.assertEqual('512 MB Server', name)
                 self.assertEqual(512, ram)
                 self.assertEqual(20, disk)
@@ -148,7 +169,7 @@ class TestNovaDriver(base.TestCase):
             #  'vcpus' : 4, 'OS-FLV-EXT-DATA:ephemeral': 0,
             #  'os-flavor-access:is_public': True, 'rxtx_factor' : 3.0,
             #  'links': {}}
-            elif id == 3:
+            elif id == '3':
                 self.assertEqual('128 MB Server', name)
                 self.assertEqual(128, ram)
                 self.assertEqual(0, disk)
@@ -159,7 +180,7 @@ class TestNovaDriver(base.TestCase):
             #  'vcpus' : 3, 'OS-FLV-EXT-DATA:ephemeral': 10,
             #  'os-flavor-access:is_public': True, 'rxtx_factor' : 2.0,
             #  'links': {}},
-            elif id == 4:
+            elif id == '4':
                 self.assertEqual('1024 MB Server', name)
                 self.assertEqual(1024, ram)
                 self.assertEqual(10, disk)
@@ -168,36 +189,49 @@ class TestNovaDriver(base.TestCase):
                 self.assertEqual(2.0, rxtx_factor)
                 self.assertEqual('1024 MB Server', name)
 
-    def test_hosts(self):
-        host_list = self.nova.hosts.list()
-        self.driver._translate_hosts(host_list)
-        host_tuples = self.driver.state[self.driver.HOSTS]
-        self.assertEqual(2, len(host_tuples))
-        # {'hosts':
-        #      [{'host_name': 'host1',
-        #        'service': 'nova-compute',
-        #        'zone': zone},
-        #       {'host_name': 'host2',
-        #        'service': 'nova-cert',
-        #        'zone': zone}]}
-        for host in host_tuples:
+#    def test_hosts(self):
+#        host_list = self.nova.hosts.list()
+#        self.driver._translate_hosts(host_list)
+#        host_tuples = self.driver.state[self.driver.HOSTS]
+#        self.assertEqual(2, len(host_tuples))
+#        for host in host_tuples:
+#            host_name = host[0]
+#            service = host[1]
+#            zone = host[2]
+#
+#            if host_name == 'host1':
+#                self.assertEqual('nova-compute', service)
+#                self.assertEqual('nova1', str(zone))
+#            elif host_name == 'host2':
+#                self.assertEqual('nova-cert', service)
+#                self.assertEqual('nova1', str(zone))
+
+    def test_hypervisors(self):
+        hypervisors = self.nova.hypervisors.list()
+        self.driver._translate_hypervisors(hypervisors)
+        hypervisor_tuples = self.driver.state[self.driver.HYPERVISORS]
+        self.assertEqual(2, len(hypervisor_tuples))
+        for host in hypervisor_tuples:
             host_name = host[0]
-            service = host[1]
-            zone = host[2]
+            id = host[1]
+            state = host[2]
+            status = host[3]
 
             if host_name == 'host1':
-                self.assertEqual('nova-compute', service)
-                self.assertEqual('nova1', str(zone))
+                self.assertEqual(2, id)
+                self.assertEqual('up', state)
+                self.assertEqual('enabled', status)
             elif host_name == 'host2':
-                self.assertEqual('nova-cert', service)
-                self.assertEqual('nova1', str(zone))
+                self.assertEqual(3, id)
+                self.assertEqual('down', state)
+                self.assertEqual('enabled', status)
 
     def test_services(self):
         service_list = self.nova.services.list()
         self.driver._translate_services(service_list)
         expected_ret = {
             1: [1, 'nova-compute', 'nova', 'nova1', 'enabled', 'up',
-                '2015-07-28T08:28:37.000000', 'None'],
+                '2015-07-28T08:28:37.000000', None],
             2: [2, 'nova-schedule', 'nova', 'nova1', 'disabled', 'up',
                 '2015-07-28T08:28:38.000000', 'daily maintenance']
         }
@@ -301,25 +335,14 @@ class TestNovaDriver(base.TestCase):
         self.assertEqual(expected_ans, nova_client.testkey)
 
     def test_execute_servers_set_meta(self):
-        class server(object):
-            def __init__(self):
-                self.testkey = None
-
-            def set_meta(self, server=None, metadata=None):
-                self.testkey = 'server=%s, metadata=%s' % (server, metadata)
-
-        class NovaClient(object):
-            def __init__(self):
-                self.servers = server()
-
-        nova_client = NovaClient()
-        self.driver.nova_client = nova_client
-        expected_ans = "server=1, metadata={'meta-key1': 'meta-value1'}"
-
-        action_args = {'positional': ['1', 'meta-key1', 'meta-value1']}
-        self.driver.execute('servers_set_meta', action_args)
-
-        self.assertEqual(expected_ans, nova_client.servers.testkey)
+        args = {'positional': ['1', 'meta-key1', 'meta-value1']}
+        action_args = {'named': {'server': '1',
+                                 'metadata': {'meta-key1': 'meta-value1'}}}
+        with mock.patch.object(self.driver, '_execute_api') as mock_ea:
+            self.driver.servers_set_meta(args)
+            mock_ea.assert_called_with(self.driver.nova_client,
+                                       'servers.set_meta',
+                                       action_args)
 
     def test_execute_with_non_executable_method(self):
         action_args = {'positional': ['1', 'meta-key1', 'meta-value1']}
