@@ -30,6 +30,7 @@ from congress.datalog import utility
 from congress.db import db_policy_rules
 from congress import exception
 from congress.policy_engines import agnostic
+from congress.synchronizer import policy_rule_synchronizer as synchronizer
 from congress.tests import base
 from congress.tests import helper
 
@@ -134,6 +135,23 @@ class TestRuntime(base.TestCase):
             'p(1)',
             'Multipolicy deletion select')
 
+    def test_cross_policy_rule(self):
+        """Test rule that refer to table from another policy."""
+        run = agnostic.Runtime()
+        run.create_policy('test1')
+        run.create_policy('test2')
+        run.create_policy('test3')
+        run.insert(
+            'p(x) :- test1:q(x),test2:q(x),test3:q(x),q(x)  q(1) q(2) q(3)',
+            'test3')
+        run.insert('q(1)', 'test1')
+        run.insert('q(1) q(2)', 'test2')
+
+        self.assertEqual(
+            run.select('p(x)', 'test3'),
+            'p(1)',
+            'Cross-policy rule select')
+
     def test_policy_types(self):
         """Test types for multiple policies."""
         # policy types
@@ -182,7 +200,8 @@ class TestRuntime(base.TestCase):
         self.assertEqual(run.select('q(5)'), '')
 
     def test_get_tablename(self):
-        run = agnostic.Runtime()
+        run = agnostic.DseRuntime('dse')
+        run.synchronizer = mock.MagicMock()
         run.create_policy('test')
         run.insert('p(x) :- q(x)')
         run.insert('q(x) :- r(x)')
@@ -217,11 +236,20 @@ class TestRuntime(base.TestCase):
         tables = run.tablenames(include_modal=False)
         self.assertEqual({'p', 'q', 'r', 's', 't'}, set(tables))
 
+    @mock.patch.object(db_policy_rules, 'add_policy')
+    def test_persistent_create_policy(self, mock_add):
+        run = agnostic.Runtime()
+        policy_name = 'invalid-table-name'
+        self.assertRaises(exception.PolicyException,
+                          run.persistent_create_policy,
+                          policy_name)
+        self.assertNotIn(policy_name, run.policy_names())
+
     @mock.patch.object(db_policy_rules, 'add_policy', side_effect=Exception())
     def test_persistent_create_policy_with_db_exception(self, mock_add):
         run = agnostic.Runtime()
-        with mock.patch.object(run, 'delete_policy') as mock_delete,\
-                mock.patch.object(run, 'synchronize_policies') as mock_sync:
+        with mock.patch.object(run, 'delete_policy') as mock_delete:
+            run.synchronizer = mock.MagicMock()
             policy_name = 'test_policy'
             self.assertRaises(exception.PolicyException,
                               run.persistent_create_policy,
@@ -231,12 +259,49 @@ class TestRuntime(base.TestCase):
                                              policy_name[:5],
                                              mock.ANY,
                                              'user',
-                                             'nonrecursive')
+                                             'nonrecursive',
+                                             session=mock.ANY)
             # mock_delete.assert_called_once_with(policy_name)
             self.assertFalse(mock_delete.called)
-            self.assertFalse(mock_sync.called)
+            self.assertFalse(run.synchronizer.sync_one_policy.called)
             self.assertNotIn('test_policy', run.policy_names())
 
+    mock_db_policy_obj = lambda: None
+    setattr(mock_db_policy_obj, 'name', 'test_policy')
+
+    @mock.patch.object(db_policy_rules, 'add_policy_rule')
+    @mock.patch.object(db_policy_rules, 'policy_name',
+                       side_effect=lambda x, session: x)
+    @mock.patch.object(
+        db_policy_rules, 'get_policies', return_value=[mock_db_policy_obj])
+    def test_persistent_insert_rules(
+            self, mock_add, mock_policy_name, mock_get_policies):
+        run = agnostic.Runtime()
+        run.synchronizer = mock.MagicMock()
+        run.create_policy('test_policy')
+
+        # test empty insert
+        result, _ = run.persistent_insert_rules('test_policy', [])
+        self.assertEqual(len(result), 0)
+        self.assertTrue(helper.datalog_equal(
+            run.select('p(x)'), ''))
+
+        # test empty string passed in rule
+        self.assertRaises(exception.PolicyException,
+                          run.persistent_insert_rules,
+                          'test_policy', [{'rule': '', 'name': '',
+                                           'comment': ''}])
+
+        # test duplicated insert, 3 rules, 2 unique
+        result, _ = run.persistent_insert_rules(
+            'test_policy',
+            [{'rule': 'p(1)', 'name': '', 'comment': ''},
+             {'rule': 'p(2)', 'name': '', 'comment': ''},
+             {'rule': 'p(1)', 'name': '', 'comment': ''}])
+        self.assertEqual(len(result), 2)
+        self.assertTrue(helper.datalog_equal(
+            run.select('p(x)'), 'p(1) p(2)'))
+
     def test_tablenames_theory_name(self):
         run = agnostic.Runtime()
         run.create_policy('test')
@@ -516,17 +581,39 @@ class TestTriggers(base.TestCase):
         run.insert('p(1)')
         self.assertEqual(0, obj.value)
 
-    def test_batch_change(self):
+    def test_batch_change_succeed(self):
         obj = self.MyObject()
         run = agnostic.Runtime()
         run.create_policy('test')
         run.register_trigger('p', lambda tbl, old, new: obj.increment())
         p1 = compile.parse1('p(1)')
-        result = run.update([compile.Event(p1, target='test')])
+        p2 = compile.parse1('p(2)')
+        p3 = compile.parse1('p(3)')
+        result = run.update([compile.Event(p1, target='test'),
+                             compile.Event(p2, target='test'),
+                             compile.Event(p3, target='test')])
         self.assertTrue(result[0], ("Update failed with errors: " +
                                     ";".join(str(x) for x in result[1])))
+        # IMPORTANT: 3 tuples inserted into p in a single batch triggers once
         self.assertEqual(1, obj.value)
 
+    def test_batch_change_fail(self):
+        obj = self.MyObject()
+        run = agnostic.Runtime()
+        run.create_policy('test')
+        run.register_trigger('p', lambda tbl, old, new: obj.increment())
+        p1 = compile.parse1('p(1)')
+        p2 = compile.parse1('p(x) :- q(x)')
+        p3 = compile.parse1('q(x) :- p(x)')
+        result = run.update([compile.Event(p1, target='test'),
+                             compile.Event(p2, target='test'),
+                             compile.Event(p3, target='test')])
+        self.assertFalse(result[0],
+                         ("Update should have failed with recursion: " +
+                          ";".join(str(x) for x in result[1])))
+        # IMPORTANT: trigger not activated even though initial events succeed
+        self.assertEqual(0, obj.value)
+
     def test_dependency(self):
         obj = self.MyObject()
         run = agnostic.Runtime()
@@ -891,6 +978,26 @@ class TestMultipolicyRules(base.TestCase):
         self.assertEqual(len(errors), 1)
         self.assertIn("Rules are recursive", str(errors[0]))
 
+    def test_dependency_graph_policy_deletion(self):
+        run = agnostic.Runtime()
+        g = run.global_dependency_graph
+        run.create_policy('test')
+        rule = 'execute[nova:flavors.delete(id)] :- nova:flavors(id)'
+        permitted, changes = run.insert(rule, target='test')
+        self.assertTrue(permitted)
+        run.create_policy('nova')
+        run.insert('flavors(1)', target="nova")
+        run.insert('flavors(2)', target="nova")
+        run.insert('flavors(3)', target="nova")
+        run.insert('flavors(4)', target="nova")
+
+        self.assertEqual(g.dependencies('test:nova:flavors.delete'),
+                         set(['nova:flavors', 'test:nova:flavors.delete']))
+        run.delete_policy('nova')
+        self.assertTrue(g.node_in('nova:flavors'))
+        self.assertEqual(g.dependencies('test:nova:flavors.delete'),
+                         set(['nova:flavors', 'test:nova:flavors.delete']))
+
     def test_dependency_graph(self):
         """Test that dependency graph gets updated correctly."""
         run = agnostic.Runtime()
@@ -974,9 +1081,9 @@ class TestMultipolicyRules(base.TestCase):
             rule.head.table.table, run.policy_object(policy).schema)
         run.insert(rulestr, policy)
         run.delete(rulestr, policy)
-        self.assertFalse(rule in run.policy_object(policy))
-        self.assertFalse(
-            rule.head.table.table in run.policy_object(policy).schema)
+        self.assertNotIn(rule, run.policy_object(policy))
+        self.assertNotIn(
+            rule.head.table.table, run.policy_object(policy).schema)
 
 
 class TestSelect(base.TestCase):
@@ -1465,6 +1572,19 @@ class TestActionExecution(base.TestCase):
         self.assertEqual(expected_args, args)
         self.assertEqual(expected_kwargs, kwargs)
 
+    def test_insert_multiple_rules(self):
+        # basic test
+
+        # test recursion caused at nth rule
+
+        # test transactional trigger activation:
+        # e.g.
+        # a(1)
+        # trigger(x) :- a(x), not b(x)
+        # b(x) :- a(x)
+        # trigger should not be activated
+        pass
+
     def test_disjunction(self):
         run = self.run
         run.create_policy('test')
@@ -1551,7 +1671,9 @@ class TestDisabledRules(base.SqlTestCase):
         late-arriving schema when importing rules already in DB.
         This behavior is not necessary in persistent_insert.
         """
-        run = agnostic.Runtime()
+        run = agnostic.DseRuntime('dse')
+        run.synchronizer = synchronizer.PolicyRuleSynchronizer(
+            run, run.node)
         run.create_policy('data', kind=datalog_base.DATASOURCE_POLICY_TYPE)
         run.persistent_create_policy('policy')
         obj = run.policy_object('policy')
